//
//  PRCrop.m
//  PRICE
//
//  Created by Riccardo Mottola on Fri Jan 28 2005.
//  Copyright (c) 2005-2009 Carduus. All rights reserved.
//
// This application is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
// This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

#include "PRCrop.h"

@implementation PRCrop

- (PRImage *)filterImage:(PRImage *)image with:(NSArray *)parameters progressPanel:(PRCProgress *)progressPanel
{
    int pixTop;
    int pixBottom;
    int pixRight;
    int pixLeft;

    /* interpret the parameters */
    pixTop = [[parameters objectAtIndex:0] intValue];
    pixBottom = [[parameters objectAtIndex:1] intValue];
    pixLeft = [[parameters objectAtIndex:2] intValue];
    pixRight = [[parameters objectAtIndex:3] intValue];

    return [self cropImage:image :pixTop :pixBottom :pixLeft :pixRight];
}

- (NSString *)actionName
{
    return @"Crop";
}

- (PRImage *)cropImage :(PRImage *)srcImage :(int)pixTop :(int)pixBottom :(int)pixLeft :(int)pixRight
{
    NSBitmapImageRep *srcImageRep;
    PRImage *destImage;
    NSBitmapImageRep *destImageRep;
    int origW, origH;
    int newW, newH;
    int x, y;
    int i;
    unsigned char *srcData;
    unsigned char *destData;
    int srcSamplesPerPixel;
    int destSamplesPerPixel;
    int srcBytesPerRow;
    int destBytesPerRow;
    BOOL isColor;
    
    /* some trace */
    NSLog(@"top: %d left:%d", pixTop, pixLeft);
    

    /* get source image representation and associated information */
    srcImageRep = [srcImage bitmapRep];
    origW = [srcImageRep pixelsWide];
    origH = [srcImageRep pixelsHigh];
    srcBytesPerRow = [srcImageRep bytesPerRow];
    srcSamplesPerPixel = [srcImageRep samplesPerPixel];
    destSamplesPerPixel = srcSamplesPerPixel;
    
    
    newW = origW - pixLeft - pixRight;
    newH = origH - pixTop - pixBottom;
    
    /* check bith depth and color/greyscale image */
    if ([srcImageRep hasAlpha])
    {
        if ([srcImageRep samplesPerPixel] == 2)
            isColor = NO;
        else
            isColor = YES;
    }
    else
    {
        if ([srcImageRep samplesPerPixel] == 1)
            isColor = NO;
        else
            isColor = YES;
    }
    
    /* allocate destination image and its representation */
    destImage = [[PRImage alloc] initWithSize:NSMakeSize(newW, newH)];
    if (isColor)
    {
        destImageRep = [[NSBitmapImageRep alloc]
                    initWithBitmapDataPlanes:NULL
                    pixelsWide:newW
                    pixelsHigh:newH
                    bitsPerSample:8
                    samplesPerPixel:destSamplesPerPixel
                    hasAlpha:NO
                    isPlanar:NO
                    colorSpaceName:NSCalibratedRGBColorSpace
                    bytesPerRow:newW*destSamplesPerPixel
                    bitsPerPixel:0];
    } else
    {
        destImageRep = [[NSBitmapImageRep alloc]
                    initWithBitmapDataPlanes:NULL
                    pixelsWide:newW
                    pixelsHigh:newH
                    bitsPerSample:8
                    samplesPerPixel:destSamplesPerPixel
                    hasAlpha:NO
                    isPlanar:NO
                    colorSpaceName:NSCalibratedWhiteColorSpace
                    bytesPerRow:newW*destSamplesPerPixel
                    bitsPerPixel:0];
    }
    srcData = [srcImageRep bitmapData];
    destData = [destImageRep bitmapData];
    destBytesPerRow = [destImageRep bytesPerRow];
    

    for (y = 0; y < newH; y++)
        for (x = 0; x < newW; x++)
            for (i = 0; i < srcSamplesPerPixel; i++)
                destData[destBytesPerRow * y + destSamplesPerPixel * x + i] = srcData[srcBytesPerRow * (y + pixTop) + srcSamplesPerPixel * (x + pixRight) + i];

    [destImage addRepresentation:destImageRep];
    [destImageRep release];
    [destImage autorelease];
    return destImage;

}

@end
